<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace HPSERVERHARDWAREModule\Lib\InitialDatabase;

use App\Model\Table\CommandsTable;
use App\Model\Table\ServicetemplatesTable;
use App\Model\Table\WizardAssignmentsTable;
use itnovum\openITCOCKPIT\InitialDatabase\Importer;

/**
 * Class TemplateImport
 * @package HPSERVERHARDWAREModule\Lib\InitialDatabase
 */
class TemplateImport extends Importer {

    /**
     * @var CommandsTable
     */
    private $CommandsTable;

    /**
     * @var ServicetemplatesTable
     */
    private $ServicetemplatesTable;


    /**
     * @var WizardAssignmentsTable
     */
    private $WizardAssignmentsTable;

    /**
     * @var array
     */
    private $mapping = [];

    /**
     * TemplateImport constructor.
     * @param CommandsTable $CommandsTable
     * @param ServicetemplatesTable $ServicetemplatesTable
     * @param WizardAssignmentsTable $WizardAssignmentsTable
     */
    public function __construct(CommandsTable $CommandsTable, ServicetemplatesTable $ServicetemplatesTable, WizardAssignmentsTable $WizardAssignmentsTable) {
        $this->CommandsTable = $CommandsTable;
        $this->ServicetemplatesTable = $ServicetemplatesTable;
        $this->WizardAssignmentsTable = $WizardAssignmentsTable;
    }


    /**
     * @return bool
     */
    public function import() {
        $data = $this->getData();
        foreach ($data['Commands'] as $command) {
            if (isset($command['uuid']) && !$this->CommandsTable->existsByUuid($command['uuid'])) {
                $entity = $this->CommandsTable->newEntity($command);
                $this->CommandsTable->save($entity);
            }
        }
        $this->mapping['servicetemplates'] = [];

        foreach ($data['Servicetemplates'] as $servicetemplate) {
            if ($this->ServicetemplatesTable->existsByUuid($servicetemplate['uuid'])) {
                $existingServicetemplate = $this->ServicetemplatesTable->getServicetemplateByUuid($servicetemplate['uuid']);
                $this->mapping['servicetemplates'][$servicetemplate['uuid']] = $existingServicetemplate['Servicetemplate']['id'];
                //skip current template if it already exists
                continue;
            }
            if (isset($servicetemplate['uuid']) && !$this->ServicetemplatesTable->existsByUuid($servicetemplate['uuid'])) {
                if (isset($servicetemplate['command_id']) && $this->CommandsTable->existsByUuid($servicetemplate['command_id'])) {
                    $command = $this->CommandsTable->getCommandByUuid($servicetemplate['command_id'], true, false)[0];
                    $servicetemplate['command_id'] = $command['id'];
                    if (!empty($servicetemplate['servicetemplatecommandargumentvalues']) && !empty($command['commandarguments'])) {
                        foreach ($servicetemplate['servicetemplatecommandargumentvalues'] as $templateArgumentKey => $templateArgumentValue) {
                            $servicetemplate['servicetemplatecommandargumentvalues'][$templateArgumentKey] = [
                                'commandargument_id' => $this->getCommandArgumentIdByName($templateArgumentValue['commandargument_id'], $command['commandarguments']),
                                'value'              => $templateArgumentValue['value'],
                            ];
                        }
                    }
                    $entity = $this->ServicetemplatesTable->newEntity($servicetemplate);
                    $this->ServicetemplatesTable->save($entity);
                    $this->mapping['servicetemplates'][$entity->get('uuid')] = $entity->get('id');
                }
            }
        }

        if (!empty($data['Wizardassignment'])) {
            $wizardassignmentEntity['uuid'] = $data['Wizardassignment']['uuid'];
            $wizardassignmentEntity['type_id'] = $data['Wizardassignment']['type_id'];
            foreach ($data['Wizardassignment']['servicetemplates']['_ids'] as $servicetemplateUuid) {
                $servicetemplate = $this->ServicetemplatesTable->getServicetemplateByUuid($servicetemplateUuid);
                if (isset($servicetemplate['Servicetemplate']) && isset($servicetemplate['Servicetemplate']['id'])) {
                    $wizardassignmentEntity['servicetemplates']['_ids'][] = $servicetemplate['Servicetemplate']['id'];
                }
            }
            if (!$this->WizardAssignmentsTable->existsByUuidAndTypeId($wizardassignmentEntity['uuid'], $wizardassignmentEntity['type_id'])) {
                $entity = $this->WizardAssignmentsTable->newEntity($wizardassignmentEntity);
                $this->WizardAssignmentsTable->save($entity);
            } else {
                $existingAssignment = $this->WizardAssignmentsTable->getWizardByUuidForEdit($wizardassignmentEntity['uuid']);
                foreach (array_diff($wizardassignmentEntity['servicetemplates']['_ids'], $existingAssignment['servicetemplates']['_ids']) as $missingServicetemplateId) {
                    $existingAssignment['servicetemplates']['_ids'][] = $missingServicetemplateId;
                }
                $entity = $this->WizardAssignmentsTable->get($existingAssignment['id']);
                $entity = $this->WizardAssignmentsTable->patchEntity($entity, $existingAssignment);
                $this->WizardAssignmentsTable->save($entity);
            }
        }
        return true;
    }

    public function getData() {
        return [
            'Commands'         => $this->getCommandsData(),
            'Servicetemplates' => $this->getServicetemplatesData(),
            'Wizardassignment' => $this->getWizardAssignmentData()
        ];
    }

    /**
     * @param string $name
     * @param array $commandarguments
     * @return bool|mixed
     */
    public function getCommandArgumentIdByName(string $name, array $commandarguments) {
        foreach ($commandarguments as $commandargument) {
            if (isset($commandargument['name']) && $commandargument['name'] === $name) {
                return $commandargument['id'];
            }
        }
        return false;
    }

    /**
     * @return array
     */
    public function getWizardAssignmentData() {
        $data = [
            'uuid'             => '9493a0d1-78d3-443b-b311-eea1b690d7dc',
            'type_id'          => 'hp-server-hardware',
            'servicetemplates' => [
                '_ids' => [
                    '756d8904-3de8-4c18-b511-61b83cef01f1'
                ]
            ]
        ];
        return $data;
    }

    /**
     * @return array
     */
    public function getCommandsData() {
        $data = [
            0 => [
                'name'             => 'check_hpasm',
                'command_line'     => '$USER1$/check_hpasm --hostname $HOSTADDRESS$ $_HOSTSNMP_SETTINGS$ --perfdata=1',
                'command_type'     => '1',
                'human_args'       => null,
                'uuid'             => '8d950614-c150-443b-9496-0645b4c76a74',
                'description'      => '',
                'commandarguments' => [],
            ]
        ];

        return $data;
    }

    /**
     * @return array
     */
    public function getServicetemplatesData() {
        $data = [
            0 => [
                'uuid'                                      => '756d8904-3de8-4c18-b511-61b83cef01f1',
                'template_name'                             => 'HP_HARDWARE_HEALTH',
                'name'                                      => 'Hardware-Health',
                'container_id'                              => '1',
                'servicetemplatetype_id'                    => '1',
                'check_period_id'                           => '1',
                'notify_period_id'                          => '1',
                'description'                               => '',
                'command_id'                                => '8d950614-c150-443b-9496-0645b4c76a74',
                'check_command_args'                        => '',
                'checkcommand_info'                         => '',
                'eventhandler_command_id'                   => '0',
                'timeperiod_id'                             => null,
                'check_interval'                            => '300',
                'retry_interval'                            => '120',
                'max_check_attempts'                        => '3',
                'first_notification_delay'                  => '0',
                'notification_interval'                     => '7200',
                'notify_on_warning'                         => '1',
                'notify_on_unknown'                         => '1',
                'notify_on_critical'                        => '1',
                'notify_on_recovery'                        => '1',
                'notify_on_flapping'                        => '0',
                'notify_on_downtime'                        => '0',
                'flap_detection_enabled'                    => '0',
                'flap_detection_on_ok'                      => '0',
                'flap_detection_on_warning'                 => '0',
                'flap_detection_on_unknown'                 => '0',
                'flap_detection_on_critical'                => '0',
                'low_flap_threshold'                        => '0',
                'high_flap_threshold'                       => '0',
                'process_performance_data'                  => '1',
                'freshness_checks_enabled'                  => '0',
                'freshness_threshold'                       => '3600',
                'passive_checks_enabled'                    => '1',
                'event_handler_enabled'                     => '0',
                'active_checks_enabled'                     => '1',
                'retain_status_information'                 => '0',
                'retain_nonstatus_information'              => '0',
                'notifications_enabled'                     => '1',
                'notes'                                     => '',
                'priority'                                  => '1',
                'tags'                                      => '',
                'service_url'                               => '',
                'is_volatile'                               => '0',
                'check_freshness'                           => '0',
                'created'                                   => '2021-06-09 09:23:57',
                'modified'                                  => '2021-06-09 09:23:57',
                'servicetemplateeventcommandargumentvalues' => [],
                'servicetemplatecommandargumentvalues'      => [],
                'customvariables'                           => [],
                'servicegroups'                             => [],
                'contactgroups'                             => [],
                'contacts'                                  => [],
            ]
        ];
        return $data;
    }
}
